<?php
namespace EMS\PortalFuncionario\Geral\Repository;

use Zeedhi\Framework\Controller\Crud;
use Doctrine\ORM\EntityManager;

use EMS\PortalFuncionario\Geral\Services\EstruturaService;

class EstruturaRepository  {

    public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
    }

    public function getPontoAcesso(string $nrorg, string $nrvinculom, string $tipoEstrutura = EstruturaService::DEFAULT_ESTRUTURA_PONTO_ACESSO) {
        try {
            $sql = <<<SQL
                SELECT PA.IDOBRIGAFOTO,
                       PA.DSLATITUDE,
                       PA.DSLONGITUDE,
                       PA.VRDISTANCIAMAXREF,
                       PA.NRESTRUTURAM
                  FROM GPE_VINCULOM VM 
                  JOIN GPE_VINCULOH VH ON VH.DTMESCOMPETENC = 
                    (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH VHH WHERE VHH.NRVINCULOM = VH.NRVINCULOM AND VHH.NRORG = VH.NRORG)
                    AND VH.NRORG = VM.NRORG
                    AND VH.NRVINCULOM = VM.NRVINCULOM
                   JOIN GPE_PONTOACESSO PA ON PA.NRESTRUTURAM = VH.$tipoEstrutura
                    AND PA.NRORG = VM.NRORG
                  WHERE VM.NRVINCULOM = :P_NRVINCULOM
                        AND VM.NRORG = :P_NRORG
SQL;

            $query = $this->entityManager->getConnection()->prepare($sql);

            $query->execute(
                array(
                    "P_NRORG"             => $nrorg,
                    "P_NRVINCULOM"        => $nrvinculom
                )
            );

            $pontoAcesso = $query->fetch();

            if (empty($pontoAcesso) && $tipoEstrutura !== EstruturaService::SECONDARY_ESTRUTURA_PONTO_ACESSO) {
                $pontoAcesso = $this->getPontoAcesso($nrorg, $nrvinculom, EstruturaService::SECONDARY_ESTRUTURA_PONTO_ACESSO);
            }

            return $pontoAcesso;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getDadosEstrutura($nrestruturam, $nrorg) {
        try {
            $sql = <<<SQL
                SELECT
                    EM.*,
                    EH.*
                  FROM ESTRUTURAM EM
                  LEFT JOIN ESTRUTURAH EH 
                        ON EH.NRESTRUTURAM = EM.NRESTRUTURAM
                        AND EH.NRORG = EM.NRORG
                        AND EH.DTMESCOMPETENC = 
                            ( SELECT MAX(EHH.DTMESCOMPETENC)
                                FROM ESTRUTURAH EHH
                               WHERE EHH.NRESTRUTURAM = EH.NRESTRUTURAM
                                 AND EHH.NRORG = EH.NRORG )
                 WHERE EM.NRESTRUTURAM = :P_NRESTRUTURAM AND EM.NRORG = :P_NRORG
SQL;
            $query = $this->entityManager->getConnection()->prepare($sql);

            $query->execute(
                array(
                    "P_NRORG"               => $nrorg,
                    "P_NRESTRUTURAM"        => $nrestruturam
                )
            );

            return $query->fetchAll()[0];
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getDadosEstruturaLegal($nrvinculom, $nrorg) {
        try {
            $sql = <<<SQL
                SELECT 
                    EM.*,
                    EH.*
                FROM GPE_VINCULOM VM 
                JOIN GPE_VINCULOH VH ON VH.DTMESCOMPETENC = 
                    (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH VHH WHERE VHH.NRVINCULOM = VH.NRVINCULOM AND VHH.NRORG = VH.NRORG)
                    AND VH.NRORG = VM.NRORG
                    AND VH.NRVINCULOM = VM.NRVINCULOM
                JOIN ESTRUTURAM EM ON EM.NRORG = VM.NRORG
                    AND EM.NRESTRUTURAM = VH.NRESTRUTLEGAL
                LEFT JOIN ESTRUTURAH EH 
                    ON EH.NRESTRUTURAM = EM.NRESTRUTURAM
                    AND EH.NRORG = EM.NRORG
                    AND EH.DTMESCOMPETENC = 
                        ( SELECT MAX(EHH.DTMESCOMPETENC)
                        FROM ESTRUTURAH EHH
                        WHERE EHH.NRESTRUTURAM = EH.NRESTRUTURAM
                            AND EHH.NRORG = EH.NRORG )
                WHERE VM.NRVINCULOM = :P_NRVINCULOM
                    AND VM.NRORG = :P_NRORG
SQL;

            $query = $this->entityManager->getConnection()->prepare($sql);

            $query->execute(
                array(
                    "P_NRORG"               => $nrorg,
                    "P_NRVINCULOM"          => $nrvinculom
                )
            );

            return $query->fetchAll()[0];
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
}